
#    WEISpub - WEIS enabler for MovableType
#    Copyright (C) 2006  Cybozu Labs, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
#    USA

package WEISpub;

use strict;

use CGI qw(popup_menu);
use Crypt::RandPasswd;
use MT::App;
use MT::Permission;
use Authen::WEIS::Provider;

use base qw(MT::App);
use vars qw($VERSION);

$VERSION = '0.9';


sub init {
    my $app = shift;
    $app->SUPER::init(@_) or return;
    $app->add_methods('weisreq' => \&handle_weisreq);
    $app->{default_mode} = 'weisreq';
    $app->{requires_login} = 1;
    $app;
}

sub handle_weisreq {
    my $app = shift;
    my $q = $app->{query};
    my $weis = Authen::WEIS::Provider->new({ params => $q });
    # check params
    return $app->show_error('Invalid WEIS Request') unless $weis->is_valid;
    # send WEIS response to consumer
    if ($q->param('weblog_selector') && $app->validate_magic) {
	if (defined($q->param('mode_ok'))) {
	    my ($user, $pass) =
		get_bind_user($app,
			      URI->new($weis->ret)->host,
			      $q->param('weblog_selector'));
	    return $weis->build_response({
		user => $user,
		pass => $pass
		});
	} else {
	    return $app->redirect($weis->build_cancel_url);
	}
    }
    # show page
    my $param = {};
    $param->{poster} = URI->new($weis->ret)->host;
    $param->{ret} = $weis->ret;
    $param->{arg} = $weis->arg;
    $param->{weblog_selector} = build_weblog_selector($app);
    $app->build_page('weispub.tmpl', $param);
}

sub get_bind_user {
    my $app = shift;
    my $host = shift;
    my $blog_id = shift;
    my $user = $app->user->name . '-' . $host;
    # find or create a new user by name
    my $author =
	MT::Author->load({ name => $user, type => MT::Author::AUTHOR });
    if (! $author) {
	$author = MT::Author->new;
	$author->name($user);
	$author->type(MT::Author::AUTHOR);
	$author->set_password(Crypt::RandPasswd->letters(16, 16));
	$author->save or die $author->errstr;
    }
    if (! $author->api_password) {
	$author->api_password(Crypt::RandPasswd->letters(16, 16));
	$author->save or die $author->errstr;
    }
    # set permissions
    my $perms = MT::Permission->load({
	blog_id => $blog_id,
	author_id => $author->id
	});
    if (! $perms) {
	$perms = MT::Permission->new();
	$perms->blog_id($blog_id);
	$perms->author_id($author->id);
    }
    $perms->can_post(1);
    $perms->save or die $perms->errstr;
    # return username and API password
    return ($user, $author->api_password);
}
	
sub build_weblog_selector {
    my $app = shift;
    my %blogs = (0 => 'Select a Weblog');
    my $iter =
	$app->user->is_superuser ?
	MT::Blog->load_iter() :
	MT::Permission->load_iter({ author_id => $app->user->id });
    while (my $thing = $iter->()) {
	if ($thing->isa('MT::Permission')) {
	    next unless $thing->can_post;
	}
	my $blog = 
	    thing->isa('MT::Blog') ? $thing : MT::Blog->load($thing->blog_id);
	$blogs{$blog->id} = $blog->name;
    }
    my @values = sort(keys %blogs);
    popup_menu('weblog_selector', \@values, 0, \%blogs);
}

1;
